#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sound.h"
#include "minifmod.h"
static int sync = -1;
#ifdef FMOD
static FMOD_SYSTEM *system;
static FMOD_SOUND *fsound;
static FMOD_CHANNEL *channel;
static unsigned int fmod_sync;
#else
FMUSIC_MODULE *mod;
#endif

/*
   god damn thygrion, you make too many varying patterns for syncing ;D
 */

//legend
//-3 = empty order
//-2 = mapping ends
//-1 = change instrument
//sync_num1, sync_num2 ... sync_numN

static const signed char instrumentOrderMap[] = {
	//lead
	0,0,0,0,0,0,0,0,0,0,0,0,-1,
	//kick
	-3,1,2,3,2,4,2,4,-3,1,2,5,6,-1,
	//snare
	-3,7,8,9,8,10,-3,-3,-3,7,8,11,-3,-1,
	//end
	-2
};

static const signed short soundSync[] = {
	//lead
	0,12,32,44,64,76,96,108,-1,
	//kick_start
	112,116,122,124,-1,
	//kick
	0,2,6,12,14,20,28,32,34,38,44,46,50,51,52,60,64,66,70,76,78,84,92,96,98,102,108,110,116,124,-1,
	//kick2
	0,2,6,12,14,20,28,32,34,38,44,46,52,60,62,63,64,66,70,76,78,84,92,96,98,102,108,110,116,124,-1,
	//kick3
	0,2,6,12,14,20,28,32,34,38,44,46,52,60,62,63,64,66,70,76,78,84,92,96,98,100,104,106,110,111,112,114,116,118,120,-1,
	//kick4
	0,2,6,12,14,20,28,34,36,40,42,46,47,48,50,52,54,56,64,68,70,76,78,84,92,96,98,102,108,110,114,115,116,124,-1,
	//kick5
	0,2,4,6,-1,
	//snare_start (8)
	111,126,-1,
	//snare
	8,24,40,54,56,62,72,88,104,120,-1,
	//snare2
	8,18,24,40,56,72,88,104,114,120,126,127,-1,
	//snare3
	8,18,24,40,56,72,88,94,96,100,102,106,108,112,114,-1,
	//snare4
	8,24,30,32,36,38,42,44,48,50,
	//ending char of the sync
	-2
};


#ifndef FMOD
//Since we're using MiniFMOD the song is
//going to be loaded from song.h' "char * data" variable
#include "song.h"

typedef struct
{
	int length;
	int pos;
	void *data;
} MEMFILE;

static unsigned int memopen(char *name)
{
	MEMFILE *memfile;
	memfile = (MEMFILE *)calloc(sizeof(MEMFILE),1);

	memfile->length = songlength;
	memfile->data = calloc(memfile->length,1);
	memcpy(memfile->data, (void*)song, memfile->length);
	memfile->pos = 0;

	return((unsigned int)memfile);
}

static void memclose(unsigned int handle)
{
	MEMFILE *memfile = (MEMFILE *)handle;
	free(memfile->data);
	free(memfile);
}

static int memread(void *buffer, int size, unsigned int handle)
{
	MEMFILE *memfile = (MEMFILE *)handle;

	if (memfile->pos + size >= memfile->length)
	{
		size = memfile->length - memfile->pos;
	}

	memcpy(buffer, (char *)memfile->data+memfile->pos, size);
	memfile->pos += size;

	return(size);
}

static void memseek(unsigned int handle, int pos, signed char mode)
{
	MEMFILE *memfile = (MEMFILE *)handle;

	switch(mode)
	{
	case SEEK_SET:
		memfile->pos = pos;
		break;
	case SEEK_CUR:
		memfile->pos += pos;
		break;
	case SEEK_END:
		memfile->pos = memfile->length + pos;
		break;
	}

	if (memfile->pos > memfile->length)
	{
		memfile->pos = memfile->length;
	}
}

static int memtell(unsigned int handle)
{
	MEMFILE *memfile = (MEMFILE *)handle;
	return(memfile->pos);
}
#endif

int sound_getOrder(void)
{
#ifdef FMOD
	FMOD_Channel_GetPosition(channel, &fmod_sync, FMOD_TIMEUNIT_MODORDER);
	return fmod_sync;
#else
	return FMUSIC_GetOrder(mod);
#endif
}

int sound_getRow(void)
{
#ifdef FMOD
	FMOD_Channel_GetPosition(channel, &fmod_sync, FMOD_TIMEUNIT_MODROW);
	return fmod_sync;
#else
	return FMUSIC_GetRow(mod);
#endif
}

int sound_isSync(int instrument)
{
	for(int ins = 0, i = 0; instrumentOrderMap[i] != -2; i++)
	{
		if (ins == instrument)
		{
			int ins_pos = instrumentOrderMap[i + sound_getOrder()];

			for(ins = 0, i = 0; soundSync[i] != -2; i++)
			{
				if ((ins == ins_pos) && (sound_getRow() == soundSync[i]) && (sync != soundSync[i]))
				{
					sync = soundSync[i];
					return 1;
				}
				else if (soundSync[i] == -1) { ins++; }
			}

			return 0;
		}
		else if (instrumentOrderMap[i] == -1) { ins++; }
	}

	return 0;
}

void sound_init(void)
{
#ifdef FMOD
	fsound = 0;
	FMOD_System_Create(&system);
	FMOD_System_Init(system, 1, FMOD_INIT_NORMAL, NULL);
	FMOD_System_CreateSound(system, "music.xm", FMOD_SOFTWARE | FMOD_CREATESTREAM, 0, &fsound);

	FMOD_System_PlaySound(system, FMOD_CHANNEL_FREE, fsound, 0, &channel);
#else
	FSOUND_File_SetCallbacks(memopen, memclose, memread, memseek, memtell);
	FSOUND_Init(44100, 0);
	mod = FMUSIC_LoadSong("music.xm", NULL);

	FMUSIC_PlaySong(mod);
#endif
}

void sound_close(void)
{
#ifdef FMOD
	FMOD_Sound_Release(fsound);
	FMOD_System_Close(system);
	FMOD_System_Release(system);
#else
	FMUSIC_FreeSong(mod);
	FSOUND_Close();
#endif
}
