// Diffusion-Limited Aggregation

#include "dla.h"

#define dlanum 25000
#define gridsize 110

float *dlapx;
float *dlapy;
float *dlapz;
float *dlapr;
float *dlapg;
float *dlapb;
float *dlaalpha;

void initdla() {
	srand(0);
	dlapx = new float[dlanum];
	dlapy = new float[dlanum];
	dlapz = new float[dlanum];
	dlapr = new float[dlanum];
	dlapg = new float[dlanum];
	dlapb = new float[dlanum];
	dlaalpha = new float[dlanum];
	int i,x,y,z;
	int tx,ty,tz;
	float xd,yd,zd,d,d2,md;
	float s = .6;
	bool ***grid;
	grid = new bool**[gridsize];
	for(y = 0; y < gridsize; y++) {
		grid[y] = new bool*[gridsize];
		for(x = 0; x < gridsize; x++) {
			grid[y][x] = new bool[gridsize];
		}
	}
	for(z = 0; z < gridsize; z++) {
		for(y = 0; y < gridsize; y++) {
			for(x = 0; x < gridsize; x++) {
				grid[z][y][x] = false;
			}
		}
	}
	grid[gridsize >> 1][gridsize >> 1][gridsize >> 1] = true;
	/*xd = (float) (gridsize >> 1);
	   yd = xd;
	   zd = xd;
	   md = sqrt(xd * xd + yd * yd + zd * zd);*/
	md = (float) (gridsize >> 1) * s;
	for(i = 0; i < dlanum; i++) {
		tx = rand() % gridsize;
		ty = rand() % gridsize;
		tz = rand() % gridsize;
		do {
			xd = (float) (tx - (gridsize >> 1)) * s;
			yd = (float) (ty - (gridsize >> 1)) * s;
			zd = (float) (tz - (gridsize >> 1)) * s;
			//if(sqrt(xd * xd + yd * yd + zd * zd) < md) {
			if(tx > 0 && ty > 0 && tz > 0 && tx < gridsize - 1 && ty < gridsize - 1 && tz < gridsize - 1) {
				if( grid[tz - 1][ty - 1][tx - 1] ||
				    grid[tz - 1][ty - 1][tx] ||
				    grid[tz - 1][ty - 1][tx + 1] ||
				    grid[tz - 1][ty][tx - 1] ||
				    grid[tz - 1][ty][tx] ||
				    grid[tz - 1][ty][tx + 1] ||
				    grid[tz - 1][ty + 1][tx - 1] ||
				    grid[tz - 1][ty + 1][tx] ||
				    grid[tz - 1][ty + 1][tx + 1] ||
				    grid[tz][ty - 1][tx - 1] ||
				    grid[tz][ty - 1][tx] ||
				    grid[tz][ty - 1][tx + 1] ||
				    grid[tz][ty][tx - 1] ||
				    grid[tz][ty][tx + 1] ||
				    grid[tz][ty + 1][tx - 1] ||
				    grid[tz][ty + 1][tx] ||
				    grid[tz][ty + 1][tx + 1] ||
				    grid[tz + 1][ty - 1][tx - 1] ||
				    grid[tz + 1][ty - 1][tx] ||
				    grid[tz + 1][ty - 1][tx + 1] ||
				    grid[tz + 1][ty][tx - 1] ||
				    grid[tz + 1][ty][tx] ||
				    grid[tz + 1][ty][tx + 1] ||
				    grid[tz + 1][ty + 1][tx - 1] ||
				    grid[tz + 1][ty + 1][tx] ||
				    grid[tz + 1][ty + 1][tx + 1]) {
					/*if( grid[tz - 1][ty][tx] ||
					    grid[tz][ty - 1][tx] ||
					    grid[tz][ty][tx - 1] ||
					    grid[tz][ty][tx + 1] ||
					    grid[tz][ty + 1][tx] ||
					    grid[tz + 1][ty][tx]) {*/
					grid[tz][ty][tx] = true;
					dlapx[i] = xd;
					dlapy[i] = yd;
					dlapz[i] = zd;
					d = 1.0 - sqrt(xd * xd + yd * yd + zd * zd) / md;
					if(d < 0) d = 0;
					if(d > 1.0) d = 1.0;
					d2 = 1.0 - d;
					dlapr[i] = d * .2 + d2;
					dlapg[i] = d2;
					dlapb[i] = d * .5 + d2;
					dlaalpha[i] = d;
					break;
				}
			} else {
				if(tx < 0) tx = 0;
				if(tx >= gridsize) tx = gridsize - 1;
				if(ty < 0) ty = 0;
				if(ty >= gridsize) ty = gridsize - 1;
				if(tz < 0) tz = 0;
				if(tz >= gridsize) tz = gridsize - 1;
				grid[tz][ty][tx] = true;
				dlapx[i] = xd;
				dlapy[i] = yd;
				dlapz[i] = zd;
				d = 1.0 - sqrt(xd * xd + yd * yd + zd * zd) / md;
				if(d < 0) d = 0;
				if(d > 1.0) d = 1.0;
				d2 = 1.0 - d;
				dlapr[i] = d * .2 + d2;
				dlapg[i] = d2;
				dlapb[i] = d * .5 + d2;
				dlaalpha[i] = d;
				break;
			}
			switch(rand() % 6) {
			case 0: tx -= 1; break;
			case 1: tx += 1; break;
			case 2: ty -= 1; break;
			case 3: ty += 1; break;
			case 4: tz -= 1; break;
			case 5: tz += 1; break;
			}
		} while(true);
	}
}

void freedla() {
	delete [] dlapx;
	delete [] dlapy;
	delete [] dlapz;
	delete [] dlapr;
	delete [] dlapg;
	delete [] dlapb;
	delete [] dlaalpha;
}

void dla(float time) {
	float et = time * 24.6;//timer_gettime();
	float dlaan = et * 8.0;
	float an2 = dlaan * 146.0;
	if(an2 > dlanum + 2000) an2 -= (an2 - (float) (dlanum + 2000)) * 20.0;
	if(an2 <= 0) return;
	float a,a2;
	int mi = (int) an2;
	if(mi > dlanum) mi = dlanum;
	glLoadIdentity();
	glBlendFunc(GL_SRC_ALPHA,GL_ONE);//_MINUS_SRC_ALPHA);
	glFogf(GL_FOG_START,15.0);
	glFogf(GL_FOG_END,150.0);
	glEnable(GL_BLEND);
	glEnable(GL_FOG);
	glDisable(GL_DEPTH_TEST);
	glTranslatef(0,0,-70.0 + sin(dlaan * .067) * 10.0);
	if(time >= .5) dlaan += 180.0;
	glRotatef(sin(dlaan * .021) * 10.0,1.0,0,0);
	glRotatef(dlaan * .67,0,1.0,0);
	glRotatef(sin(dlaan * .034) * 8.0,0,0,1.0);
	glPointSize((float) getscrw() / 1024.0 * 5.0);
	glBegin(GL_POINTS);
	for(int i = 0; i < mi; i++) {
		a = dlaalpha[i];
		if(i == mi - 1) {
			a *= an2 - (float) mi;
			dlaalpha[i] = a;
		}
		a2 = a * 1.5;
		if(a2 > 1.0) a2 = 1.0;
		glColor4f(dlapr[i],dlapg[i],dlapb[i],a2);
		glVertex3f(dlapx[i],dlapy[i],dlapz[i]);
	}
	glEnd();
	glPointSize((float) getscrw() / 1024.0 * 15.0);
	glBegin(GL_POINTS);
	for(int i = 0; i < mi; i++) {
		a = dlaalpha[i];
		glColor4f(dlapr[i],dlapg[i],dlapb[i],a * .3);
		glVertex3f(dlapx[i],dlapy[i],dlapz[i]);
	}
	glEnd();
	glDisable(GL_BLEND);
	glDisable(GL_FOG);
	glEnable(GL_DEPTH_TEST);
}
