#include "start.h"
#include "../general.h"

typedef struct
{
	float x, y;
} point;

static const unsigned char letterconf[] = {
	5,9,5,5,9,13,13,5,5,5,5,7
};

static const point letters[] = {
	//i
	{50,270},
	{50,370},
	{70,370},
	{70,270},
	{50,270},
	//c
	{80,270},
	{80,370},
	{180,370},
	{180,350},
	{100,350},
	{100,290},
	{180,290},
	{180,270},
	{80,270},
	//o
	{190,270},
	{190,370},
	{290,370},
	{290,270},
	{190,270},
	{210,290},
	{210,350},
	{270,350},
	{270,290},
	{210,290},
	//n
	{300,270},
	{300,370},
	{400,370},
	{400,270},
	{380,270},
	{380,350},
	{320,350},
	{320,270},
	{300,270},
	//s
	{410,270},
	{510,270},
	{510,330},
	{430,330},
	{430,350},
	{510,350},
	{510,370},
	{410,370},
	{410,310},
	{490,310},
	{490,290},
	{410,290},
	{410,270},

	//2
	{110,160},
	{210,160},
	{210,180},
	{130,180},
	{130,200},
	{210,200},
	{210,260},
	{110,260},
	{110,240},
	{190,240},
	{190,220},
	{110,220},
	{110,160},
	//0
	{220,160},
	{220,260},
	{320,260},
	{320,160},
	{220,160},
	{240,240},
	{300,240},
	{300,180},
	{240,180},
	{240,240},
	//0
	{330,160},
	{330,260},
	{430,260},
	{430,160},
	{330,160},
	{350,180},
	{350,240},
	{410,240},
	{410,180},
	{350,180},
	//7
	{440,260},
	{540,260},
	{540,160},
	{520,160},
	{520,240},
	{440,240},
	{440,260}
};

static const int dna_tri_max = 100;
static void dna(float time)
{
	int dna_tri = 100, i = 0;
	if ((time > 0.25) && (time <= 0.5))
	{
		dna_tri = (int)(dna_tri_max*((time-0.25)/0.25));
	}
	else if (time <= 0.25) { dna_tri = 0; }
	else if (time > 0.75) { i = (int)(dna_tri_max*((time-0.75)/0.25)); }

	glEnable(GL_BLEND);
	//Triangle DNA
	glDisable(GL_DEPTH_TEST);
	glColor3f(0.1,0.1,0.1);

	float moot=70*time;
	float angle;
	glBegin(GL_TRIANGLES);
	for(; i<dna_tri; i++)
	{
		angle=2*M_PI*(i/30.0f);
		glVertex3f(sin(angle+moot), -5+i*0.1f,-5+cos(angle+moot));

		glVertex3f((1-sin(angle+moot)), -5+(i)*0.1f,-5+(1-cos(angle+moot)));
	}
	glEnd();
	glEnable(GL_DEPTH_TEST);
}

void start_run(float time)
{
	float alpha = 1.0;
	if(time < .2) alpha = time * 5.0;

	perspective2DBegin(640, 480);
	glColor3f(0.91 * alpha,0.0,0.52 * alpha);

	float time1 = time, time2 = 0.0f;

	if (time <= 0.25)
	{
		time1 *= 4;
	}
	else { time1 = 1.0f; }

	if (time > 0.8)
	{
		time1 = time2 = 1-(time-0.8)/0.2;
	}

	int i, step_num = 0;
	for (i = 0; i < 6; i++)
	{
		glBegin(GL_LINE_STRIP);
		int j;
		for(j = 0; j < (letterconf[i]-1)*time1; j++)
		{
			glVertex2f(letters[step_num+j].x, letters[step_num+j].y);
		}
		const float time_for_line = (time1-((1.0/((float)letterconf[i]-1))*(j-1)))/(1.0/((float)letterconf[i]-1));

		glVertex2f(
			interpolate(letters[step_num+j-1].x, letters[step_num+j].x, time_for_line),
			interpolate(letters[step_num+j-1].y, letters[step_num+j].y, time_for_line)
			);

		glEnd();
		step_num += letterconf[i];
	}

	if ((time > 0.5) && (time <= 0.75))
	{
		time2 = (time-0.5)*4;
	}
	else if (time <= 0.5f) { time2 = 0.0f; }
	else if (time <= 0.8) { time2 = 1.0f; }

	for (i = 6; i < 12; i++)
	{
		glColor4f(alpha,alpha,alpha,0);
		glBegin(GL_LINE_STRIP);
		int j;
		for(j = 0; j < (letterconf[i]-1)*time2; j++)
		{
			glVertex2f(letters[step_num+j].x, letters[step_num+j].y);
		}
		const float time_for_line = (time2-((1.0/((float)letterconf[i]-1))*(j-1)))/(1.0/((float)letterconf[i]-1));

		glVertex2f(
			interpolate(letters[step_num+j-1].x, letters[step_num+j].x, time_for_line),
			interpolate(letters[step_num+j-1].y, letters[step_num+j].y, time_for_line)
			);

		glEnd();
		step_num += letterconf[i];
	}

	perspective2DEnd();

	dna(time);
}
