#include "player.h"
#include "general.h"

#include "Effects/iterator.h"
#include "Effects/fuzzy.h"
#include "Effects/dla.h"
#include "Effects/electro.h"
#include "Effects/filler.h"
#include "Effects/start.h"
#include "Effects/tunnel.h"
#include "Effects/cred.h"

//SCENE_MAX need to be set to the amount of scenes being loaded
#define SCENE_MAX 8
//endTime is the time in seconds when the demo ends
static float endTime = 126.0f;
//to rewind demo to a certain position. NOTE: music will NOT rewind
//example: 20 seconds forward: -20.0f
float playerRewindTime = -0.0f;

typedef struct {
	const char *name;
	float start;
	float end;
	void (*init)(void);
	void (*run)(float);

	void (*deinit)(void);
} scene;

scene scenes[SCENE_MAX];

void playerInit(void)
{
	/*
	   Add all the effects here:
	   (playing order will be the addingorder if the scenes overlap)
	   playerAddScene(
	        scenename,
	        scene start time in seconds,
	        scene duration in seconds,
	        scene init function pointer,
	        scene run function pointer,
	        scener deinit function pointer
	   )
	 */

	// Start
	// Iterator
	// Electro
	// DLA
	// Tunnel
	// Fuzzy

	// I moved these a bit to get the most out of the loading bar fader.
	playerAddScene("Start", 0.0f, 20.0f, NULL, &start_run, NULL);
	playerAddScene("Filler", 0.0f, 122.0f, &filler_init, &filler_run, NULL);
	playerAddScene("Iterator", 18.5f, 21.5f, &inititerator, &iterator, &freeiterator);
	playerAddScene("Electro", 39.0f, 21.0f, &initelectro, &electro, &freeelectro);
	playerAddScene("DLA", 60.0f, 20.0f, &initdla, &dla, &freedla);
	playerAddScene("Tunnel", 80.0f, 20.0f, &tunnel_init, &tunnel_run, &tunnel_deinit);
	playerAddScene("Fuzzy", 100.0f, 20.0f, &initfuzzy, &fuzzy, &freefuzzy);
	playerAddScene("Credits",119.2,6.8,&initcred,&cred,&freecred);

	//playerAddScene("Structure", 130.0f, 20.0f, &initstructure, &structure, &freestructure);

	for(int i = 0; i < SCENE_MAX; i++)
	{
		float alpha = 1.0;

		glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
		viewReset();
		perspective2DBegin(640, 480);

		const float barLength = i/((float)(SCENE_MAX - 1));
		if(barLength > .25) alpha = 1.0 - (barLength - .25) / .75;
		//if(barLength > .5) alpha = 1.0 - (barLength - .5) / .5;
		//alpha = 1.0 - i/((float)SCENE_MAX);

		glLineWidth(2.0f);
		glBegin(GL_LINE_STRIP);
		glColor3f(alpha,alpha,alpha);
		glVertex2f(128,220);
		glVertex2f(512,220);
		glVertex2f(512,260);
		glVertex2f(128,260);
		glVertex2f(128,220);
		/*glVertex2f(133,225);
		   glVertex2f(133+374,225);
		   glVertex2f(133+374,255);
		   glVertex2f(133,255);
		   glVertex2f(133,225);*/
		glEnd();

		glBegin(GL_QUADS);
		glColor3f(0.91 * alpha,0.0,0.52 * alpha);
		glVertex2f(133,225);
		glVertex2f(133+374*barLength,225);
		glVertex2f(133+374*barLength,255);
		glVertex2f(133,255);
		glEnd();

		glLineWidth(1.0f);

		perspective2DEnd();

		SwapBuffers(hDC);

		if (scenes[i].init)
		{
			(*scenes[i].init)();
			//printf("Loaded %s\n", scenes[i].name);
		}
	}
}

static unsigned int scene_i = 0;
void playerAddScene(const char *name,
                    float start, float duration,
                    void (*init)(void), void (*run)(float), void (*deinit)(void)
                    )
{
	if (scene_i < SCENE_MAX)
	{
		scenes[scene_i].name = name;
		scenes[scene_i].start = start;
		scenes[scene_i].end = start + duration;
		scenes[scene_i].init = init;
		scenes[scene_i].run = run;
		scenes[scene_i].deinit = deinit;
		scene_i++;
	}
}

int playerRun(void)
{
	const float currentTime = timer_gettime();

	if (currentTime > endTime)
	{
		return 0;
	}

	for(int i = 0; i < SCENE_MAX; i++)
	{
		if ((currentTime > scenes[i].start) && (currentTime < scenes[i].end))
		{
			(*scenes[i].run)((currentTime - scenes[i].start) / (scenes[i].end - scenes[i].start));
#ifdef DEBUG
			if (debugSyncHit)
			{
				printf("HIT %s: %.3f\n", scenes[i].name, (currentTime - scenes[i].start) / (scenes[i].end - scenes[i].start));
			}
#endif
		}
	}

#ifdef DEBUG
	debugSyncHit = false;
#endif

	return 1;
}

void playerClose(void)
{
	for(int i = 0; i < SCENE_MAX; i++)
	{
		if (scenes[i].deinit)
		{
			(*scenes[i].deinit)();
		}
	}
}
